#ifndef PQDETECTIONSDKTOOL_H
#define PQDETECTIONSDKTOOL_H

#define DEFAULT_MODE 0
#define CALLBACK_MODE 1

#define MAX_TAG_COUNT 4
#define MAX_OBJECT_COUNT 16


#define STATUS_DOWN 0
#define STATUS_MOVE 1
#define STATUS_UP 2

enum DEVICE_STATUS
{
	DEVICE_DISCONNECT = -1, //eyesService filter ӳ
	NORMAL = 0,
	EYE_MISS = 1,
	EYE_COVER = 2
};

/// Detected object detail.
typedef struct
{
	int		id;
	int		type;					///< object type: e.g.    0,      1,      2,    ...  defined in "pq_object_defination.xml"
	char	type_description[256];	///< description: e.g.  "car", "juice", "wine" 
	float	cx;						///< object center x position. value range normalized to 0.0 ~ 1.0
	float	cy;						///< object center y position. value range normalized to 0.0 ~ 1.0
	float	w;						///< reserved. value undefined
	float	h;						///< reserved. value undefined
	int		status;					///< object status on the screen:   STATUS_DOWN		STATUS_MOVE		STATUS_UP
}ObjectDetail;

/// Detected tag detail.
typedef struct
{
	int		id;
	int		type;					///< tag type: 0, 1, 2, 3
	float	cx;						///< tag center x position. value range normalized to 0.0 ~ 1.0
	float	cy;						///< tag center y position. value range normalized to 0.0 ~ 1.0
	float	w;						///< reserved. value undefined
	float	h;						///< reserved. value undefined
	int		status;					///< tag status on the screen:   STATUS_DOWN		STATUS_MOVE		STATUS_UP
}TagDetail;

/// All detected data.
typedef struct
{
	TagDetail tags[MAX_TAG_COUNT];
	int tagsCount;

	ObjectDetail objects[MAX_OBJECT_COUNT];
	int objectsCount;
}DetectionData;

typedef struct
{
	char SN[14];
	int w;
	int h;
}DEVICE_PLATFORM_INFO;
/// Callback function definition, execute when detect objects.
typedef void(__cdecl *PFOnDetected)(DetectionData detectionData ///< [in] detected data at time
	);

#ifdef PQDETECTIONSDKTOOL_EXPORTS
#define PQDETECTIONSDKTOOL_API __declspec(dllexport)
#else
#define PQDETECTIONSDKTOOL_API __declspec(dllimport)
#endif

#ifdef __cplusplus
extern "C"
{
#endif
	///Initialize the resources required by the program and establish a connection to the platform.
	/**This function need run first when you want to use this library.
		\return 
		return 0 SUCCESS.\n
		return 1 This function is not valid for currently getting object recognition information mode by default.\n
		return 2 hostIP error.\n
		return 3 port error.\n
		return 5 pq_object_defination. XML is missing or corrupted.\n
	*/
	int PQDETECTIONSDKTOOL_API __cdecl InitAndRun(
		const char *hostIP,				///< default "127.0.0.1"
		unsigned short port,			///< default "53607"
		int callbackMode,				///< default 0
		PFOnDetected OnDetected			///< default 0
	);

	///Disconnect from the platform and release all resources used.
	/**This function need to run when you want to disconnect from platform or exit application.*/
	void PQDETECTIONSDKTOOL_API __cdecl ShutdownAndRelease();

	///In the default mode, the user actively requests to get new detect data.
	/**This function needs to be run all the time, and if it is stopped calling continuously, the platform will always wait for the function call to complete or disconnect before continuing to run.
		\return
		return: 0 SUCCESS.\n
		return: 1 pMagicEyeFrame is NULL.\n
		return: 2 is currently the object recognition information pattern obtained from the asynchronous callback function, which is invalid.\n
		return: 3 SDK is disconnected from the MultiTouchPlatform network.\n
		return: 4 no new data needs to be obtained.\n
	*/
	int PQDETECTIONSDKTOOL_API __cdecl ProcessDetectedData(DetectionData *pDetectionData ///< [out] detected data at time
	);

	/** Get PlatForm infomation serial number, width and height
		return: 0 SUCCESS.
		return: -1 get infomation error. try call this function again
	*/
	int PQDETECTIONSDKTOOL_API __cdecl GetPlatFormInfo(DEVICE_PLATFORM_INFO *deviceInfo);

#ifdef __cplusplus
}
#endif

#endif
