#include "PQDetectionSDKTool.h"
#include <stdio.h>
#include <Windows.h>
#include <unordered_set>
#include <assert.h>

#pragma comment(lib, "PQDetectionSDKTool.lib")

std::unordered_set<int> g_tagIDs, g_objectIDs;
void Check(int id, int status, std::unordered_set<int> *pIds);

void OnDetected(DetectionData detectionData)
{
	//show tags detail
	for (int i = 0; i < detectionData.tagsCount; i++)
	{
		printf("tagId: %d, \ttagType: %d, \ttagAction: %d, \tcX: %.2f, \tcY: %.2f\n", detectionData.tags[i].id, detectionData.tags[i].type, detectionData.tags[i].status, detectionData.tags[i].cx, detectionData.tags[i].cy);
		Check(detectionData.tags[i].id, detectionData.tags[i].status, &g_tagIDs);
	}
	//show objects detail
	for (int i = 0; i < detectionData.objectsCount; i++)
	{
		printf("eyesid: %d, \ttype: %d, \ttype_description: %s \tstatus: %d, \tcX: %.2f, \tcY: %.2f\n", detectionData.objects[i].id, detectionData.objects[i].type, detectionData.objects[i].type_description, detectionData.objects[i].status, detectionData.objects[i].cx, detectionData.objects[i].cy);
		printf("detectionData.objects[i].w %f detectionData.objects[i].h %f\n", detectionData.objects[i].w, detectionData.objects[i].h);
		Check(detectionData.objects[i].id, detectionData.objects[i].status, &g_objectIDs);
	}
	printf("-----------------------------------------------\r\n");
}

void Check(int id, int status, std::unordered_set<int> *pIds)
{
	std::unordered_set<int>::iterator it = pIds->find(id);
	switch (status)
	{
	case STATUS_DOWN:
		assert(it == pIds->end());
		if (it != pIds->end()) {
			printf("****** ERROR: %d down but already exist\n", id);
		}
		pIds->insert(id);
		break;
	case STATUS_MOVE:
		assert(it != pIds->end());
		if (it == pIds->end()) {
			printf("****** ERROR: %d move but NOT down\n", id);
		}
		break;
	case STATUS_UP:
		assert(it != pIds->end());
		if (it == pIds->end()) {
			printf("****** ERROR: %d up but NOT down\n", id);
		}
		else {
			pIds->erase(it);
		}
		break;
	}
}

void StartPreProcess(const char * name)
{
	STARTUPINFO si; //
	memset(&si, 0, sizeof(STARTUPINFO));
	si.cb = sizeof(STARTUPINFO);
	si.dwFlags = STARTF_USESHOWWINDOW;
	si.wShowWindow = SW_HIDE;//SW_SHOW;

	PROCESS_INFORMATION pi; //
	
	if (!CreateProcess(NULL, (LPSTR)name, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi)) {
		return;
	}
	Sleep(3000);
}

// callback mode
#if 0
int main()
{
	//const char* platform = "C:/Program Files (x86)/PQLabs/MultiTouchPlatform/MultiTouchPlatform.exe";
	const char* service = "C:/coding/magicTable/darknet/x64/Release/G7EyesService.exe"; //C:/coding/magicTable/darknet/x64/Release/
	StartPreProcess(service);
	//StartPreProcess(platform);
	int re = InitAndRun("127.0.0.1", 53607, CALLBACK_MODE, OnDetected);
	if (re != 0)
	{
		printf("InitAndRun Error. Error code:%d\r\n", re);
	}

	getchar();
	ShutdownAndRelease();
}
#endif // 0


//// non callback mode
int main()
{
	int re = InitAndRun("127.0.0.1", 53607, DEFAULT_MODE, 0);
	if (re != 0)
	{
		printf("InitAndRun Error. Error code:%d\r\n", re);
		getchar();
		return re;
	}
	DEVICE_PLATFORM_INFO devInfo = {};
	while (true)
	{
		if (GetPlatFormInfo(&devInfo) == 0)
		{
			printf("%s: w:%d h:%d\n", devInfo.SN, devInfo.w, devInfo.h);
			break;
		}
		else printf("error\n");
	}
	while (true)
	{
		
		DetectionData detectionData;
		re = ProcessDetectedData(&detectionData);
		if(re == 0)
			OnDetected(detectionData);
		//Sleep(2000);
	}
	Sleep(1000);
	printf("release begin.\n");
	ShutdownAndRelease();
	printf("done.\n");
}