﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

public enum GET_DETECTION_DATA_MODE
{
    DEFAULT_MODE = 0,
    CALLBACK_MODE = 1
}
public struct ObjectDetail
{
    public int id;
    public int type;                    // object type: e.g.    0,      1,      2,    ...  defined in "pq_object_defination.xml"
    [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256, ArraySubType = UnmanagedType.ByValArray)]
    public Byte[] type_description;     // description: e.g.  "car", "juice", "wine" 
    public float cx;					// object center x position. value range normalized to 0.0 ~ 1.0
    public float cy;					// object center y position. value range normalized to 0.0 ~ 1.0
    public float w;					    // reserved. value undefined
    public float h;					    // reserved. value undefined
    public int status;				    // object status on the screen:   STATUS_DOWN		STATUS_MOVE		STATUS_UP
};

public struct TagDetail
{
    public int id;
    public int type;				    // tag type: 0, 1, 2, 3
    public float cx;					// tag center x position. value range normalized to 0.0 ~ 1.0
    public float cy;					// tag center y position. value range normalized to 0.0 ~ 1.0
    public float w;					    // reserved. value undefined
    public float h;					    // reserved. value undefined
    public int status;				    // tag status on the screen:   STATUS_DOWN		STATUS_MOVE		STATUS_UP
};

//[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
public struct DEVICE_PLATFORM_INFO
{
    [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 14)]
    public string SN;
    public int w;
    public int h;
};

public struct DetectionData
{
    [MarshalAs(UnmanagedType.ByValArray, SizeConst = 4, ArraySubType = UnmanagedType.Struct)]
    public TagDetail[] tags;
    public int tagsCount;
    [MarshalAs(UnmanagedType.ByValArray, SizeConst = 16, ArraySubType = UnmanagedType.Struct)]
    public ObjectDetail[] objects;
    public int objectsCount;
};

public class PQDetectionSDKTool
{
    public PQDetectionSDKTool()
    {

    }

    ~PQDetectionSDKTool()
    {

    }

    #region import PQDetectionSDKTool

    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void PFOnDetected(DetectionData detectionData);

    [DllImport(@"PQDetectionSDKTool.dll", EntryPoint = "InitAndRun", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
    static extern int InitAndRun(byte[] hostIP, UInt16 port, int callbackMode, PFOnDetected OnDetected);

    [DllImport(@"PQDetectionSDKTool.dll", EntryPoint = "ShutdownAndRelease", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
    static extern void ShutdownAndRelease();

    [DllImport(@"PQDetectionSDKTool.dll", EntryPoint = "ProcessDetectedData", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
    static extern int ProcessDetectedData(ref DetectionData detectionData);

    [DllImport(@"PQDetectionSDKTool.dll", EntryPoint = "GetPlatFormInfo", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]

    static extern int GetPlatFormInfo_Internal(ref DEVICE_PLATFORM_INFO device_info);


    #endregion

    bool m_initOK = false;
    PFOnDetected m_onDetected = null;
    GET_DETECTION_DATA_MODE m_callbackMode = GET_DETECTION_DATA_MODE.CALLBACK_MODE;
    byte[] m_baHostIP = null;
    ushort m_port = 0;

    public void OnDetected(DetectionData detectionData)
    {

    }

    public int InitAndRunEx(string hostIP = "127.0.0.1", ushort port = 53607, GET_DETECTION_DATA_MODE callbackMode = GET_DETECTION_DATA_MODE.DEFAULT_MODE, PFOnDetected OnDetected = null)
    {
        int re = 0;
        if (callbackMode == GET_DETECTION_DATA_MODE.CALLBACK_MODE && OnDetected == null)
        {
            re = 1;
            goto Error;
        }
        m_callbackMode = callbackMode;
        m_onDetected = OnDetected;

        if (hostIP == null || hostIP.Length == 0)
        {
            re = 2;
            goto Error;
        }
        byte[] buHostIP = new byte[hostIP.Length * 2 + 2];
        m_baHostIP = new byte[hostIP.Length + 1];
        Encoding ascii = Encoding.ASCII;
        Encoding unicode = Encoding.Unicode;
        buHostIP = unicode.GetBytes(hostIP);
        m_baHostIP = Encoding.Convert(unicode, ascii, buHostIP);
        if (port < 1 || port > 65535)
        {
            re = 3;
            goto Error;
        }
        m_port = port;
        int re2 = InitAndRun(m_baHostIP, m_port, (int)m_callbackMode, m_onDetected);
        if (re2 != 0)
        {
            re = re2;
            goto Error;
        }

        m_initOK = true;
        Error:
        return re;
    }

    public void ShutdownAndReleaseEx()
    {
        if (m_initOK)
            ShutdownAndRelease();
        m_initOK = false;
    }

    public int ProcessDetectedFrameEx(ref DetectionData detectionData)
    {
        return ProcessDetectedData(ref detectionData);
    }

    public void InitObjectDetail(ref ObjectDetail pObjectDetail)
    {
        pObjectDetail.id = -1;
        pObjectDetail.status = -1;
        pObjectDetail.type = -1;
        if (pObjectDetail.type_description != null)
            Array.Clear(pObjectDetail.type_description, 0, 256);
        pObjectDetail.cx = 0;
        pObjectDetail.cy = 0;
        pObjectDetail.w = 0;
        pObjectDetail.h = 0;
    }

    public void InitTagDetail(ref TagDetail pTagDetail)
    {
        pTagDetail.id = -1;
        pTagDetail.status = -1;
        pTagDetail.type = -1;
        pTagDetail.cx = 0;
        pTagDetail.cy = 0;
        pTagDetail.w = 0;
        pTagDetail.h = 0;
    }

    public void InitDetectionData(ref DetectionData detectionData)
    {
        detectionData.objectsCount = 0;
        for (int i = 0; i < 16; i++)
        {
            InitObjectDetail(ref detectionData.objects[i]);
        }
        detectionData.tagsCount = 0;
        for (int i = 0; i < 4; i++)
        {
            InitTagDetail(ref detectionData.tags[i]);
        }
    }
    public int GetPlatFormInfo(ref DEVICE_PLATFORM_INFO devicePlatformInfo)
    {
        int status = -1;
        status = GetPlatFormInfo_Internal(ref devicePlatformInfo);
        return status;
    }

}